/*
             LUFA Library
     Copyright (C) Dean Camera, 2017.

  dean [at] fourwalledcubicle [dot] com
           www.lufa-lib.org
*/

/*
  Copyright 2017  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaims all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/** \file
 *  \brief Board specific Buttons driver header for the Fletchtronics BUMBLEB.
 *  \copydetails Group_Buttons_BUMBLEB
 *
 *  \note This file should not be included directly. It is automatically included as needed by the Buttons driver
 *        dispatch header located in LUFA/Drivers/Board/Buttons.h.
 */

/** \ingroup Group_Buttons
 *  \defgroup Group_Buttons_BUMBLEB BUMBLEB
 *  \brief Board specific Buttons driver header for the Fletchtronics BUMBLEB.
 *
 *  Board specific buttons driver header for the Fletchtronics BUMBLEB (http://fletchtronics.net/bumble-b). The BUMBLEB
 *  third-party board does not include any on-board peripherals, but does have an officially recommended external peripheral
 *  layout for buttons, LEDs and a Joystick.
 *
 *  <table>
 *    <tr><th>Name</th><th>Info</th><th>Active Level</th><th>Port Pin</th></tr>
 *    <tr><td>BUTTONS_BUTTON1</td><td>HWB Button</td><td>Low</td><td>PORTD.7</td></tr>
 *  </table>
 *
 *  @{
 */

#ifndef __BUTTONS_BUMBLEB_H__
#define __BUTTONS_BUMBLEB_H__

	/* Includes: */
		#include "../../../../Common/Common.h"

	/* Enable C linkage for C++ Compilers: */
		#if defined(__cplusplus)
			extern "C" {
		#endif

	/* Preprocessor Checks: */
		#if !defined(__INCLUDE_FROM_BUTTONS_H)
			#error Do not include this file directly. Include LUFA/Drivers/Board/Buttons.h instead.
		#endif

	/* Public Interface - May be used in end-application: */
		/* Macros: */
			/** Button mask for the first button on the board. */
			#define BUTTONS_SEL      (1 << 5) //PB5
			#define BUTTONS_STOP   (1 << 7) //PB7
			#define BUTTONS_BAS      (1 << 5) //PF5
			#define BUTTONS_DROITE      (1 << 4) //PF4
			#define BUTTONS_GAUCHE      (1 << 1) //PF1
			#define BUTTONS_HAUT      (1 << 0) //PF0


		/* Inline Functions: */
		#if !defined(__DOXYGEN__)
				//BUTTON SEL
				//
		
			static inline void Buttons_Init_SEL(void)
			{
				DDRB  &= ~BUTTONS_SEL;
				PORTB |=  BUTTONS_SEL;
			}

			static inline void Buttons_Disable_SEL(void)
			{
				DDRB  &= ~BUTTONS_SEL;
				PORTB &= ~BUTTONS_SEL;
			}


		
			// BUTTON STOP

			static inline void Buttons_Init_STOP(void)
			{
				DDRB  &= ~BUTTONS_STOP;
				PORTB |=  BUTTONS_STOP;
			}

			
			// BUTTON STOP

			static inline void Buttons_Disable_STOP(void)
			{
				DDRB  &= ~BUTTONS_STOP;
				PORTB |=  BUTTONS_STOP;
			}

			
			

			//BUTTON BAS


			static inline void Buttons_Init_BAS(void)
			{
				DDRF  &= ~BUTTONS_BAS;
				PORTF |=  BUTTONS_BAS;
			}

			static inline void Buttons_Disable_BAS(void)
			{
				DDRF  &= ~BUTTONS_BAS;
				PORTF &= ~BUTTONS_BAS;
			}
	
			//BUTTON DROITE

			static inline void Buttons_Init_DROITE(void)
			{
				DDRF  &= ~BUTTONS_DROITE;
				PORTF |=  BUTTONS_DROITE;
			}

			static inline void Buttons_Disable_DROITE(void)
			{
				DDRF  &= ~BUTTONS_DROITE;
				PORTF &= ~BUTTONS_DROITE;
			}

			//BUTTON GAUCHE 

			static inline void Buttons_Init_GAUCHE(void)
			{
				DDRF  &= ~BUTTONS_GAUCHE;
				PORTF |=  BUTTONS_GAUCHE;
			}

			static inline void Buttons_Disable_GAUCHE(void)

			{
				DDRF  &= ~BUTTONS_GAUCHE;
				PORTF &= ~BUTTONS_GAUCHE;
			}
			
			//BUTTON HAUT

			static inline void Buttons_Init_HAUT(void)
			{
				DDRF  &= ~BUTTONS_HAUT;
		
				//PORTF |=  BUTTONS_HAUT; pas besoin, ne sert que pour le port b apparemment faudrait enlever cette ligne pour tous les boutons sauf pour le port B
			}

			static inline void Buttons_Disable_HAUT(void)
			{
				DDRF  &= ~BUTTONS_HAUT;
				PORTF &= ~BUTTONS_HAUT;
			}

		
			//SEL
		
			static inline uint8_t Buttons_GetStatus_V(void) ATTR_WARN_UNUSED_RESULT;
			static inline uint8_t Buttons_GetStatus_V(void)
			{
				return ((PINB & BUTTONS_SEL) ^ BUTTONS_SEL);
			}

			//STOP

			static inline uint8_t Buttons_GetStatus_STOP(void) ATTR_WARN_UNUSED_RESULT;
			static inline uint8_t Buttons_GetStatus_STOP(void)
			{
				return ((PINB & BUTTONS_STOP) ^ BUTTONS_STOP);
			}
			//BAS

			static inline uint8_t Buttons_GetStatus_BAS(void) ATTR_WARN_UNUSED_RESULT;
			static inline uint8_t Buttons_GetStatus_BAS(void)
			{
				return ((PINF & BUTTONS_BAS) ^ BUTTONS_BAS);
			}

			//DROITE

			static inline uint8_t Buttons_GetStatus_DROITE(void) ATTR_WARN_UNUSED_RESULT;
			static inline uint8_t Buttons_GetStatus_DROITE(void)
			{
				return ((PINF & BUTTONS_DROITE) ^ BUTTONS_DROITE);
			}
			
			//GAUCHE

			static inline uint8_t Buttons_GetStatus_GAUCHE(void) ATTR_WARN_UNUSED_RESULT;
			static inline uint8_t Buttons_GetStatus_GAUCHE(void)
			{
				return ((PINF & BUTTONS_GAUCHE) ^ BUTTONS_GAUCHE);
			}

			//HAUT

			static inline uint8_t Buttons_GetStatus_HAUT(void) ATTR_WARN_UNUSED_RESULT;
			static inline uint8_t Buttons_GetStatus_HAUT(void)
			{
				return (PINF & BUTTONS_HAUT) ;
			}



		#endif

	/* Disable C linkage for C++ Compilers: */
		#if defined(__cplusplus)
			}
		#endif

#endif

/** @} */

